function [minimum,fval] = anneal(loss, parent, options)
% ANNEAL  Minimizes a function with the method of simulated annealing
% (Kirkpatrick et al., 1983)
%
%  ANNEAL takes three input parameters, in this order:
%
%  LOSS is a function handle (anonymous function or inline) with a loss
%  function, which may be of any type, and needn't be continuous. It does,
%  however, need to return a single value.
%
%  PARENT is a vector with initial guess parameters. You must input an
%  initial guess.
%
%  OPTIONS is a structure with settings for the simulated annealing. If no
%  OPTIONS structure is provided, ANNEAL uses a default structure. OPTIONS
%  can contain any or all of the following fields (missing fields are
%  filled with default values):
%
%       Verbosity: Controls output to the screen.
%                  0 suppresses all output
%                  1 gives final report only [default]
%                  2 gives temperature changes and final report 
%        InitTemp: The initial temperature, can be any positive number.
%                  Default is 100.
%        StopTemp: Temperature at which to stop, can be any positive number
%                  smaller than InitTemp. 
%                  Default is 1e-2.
%         StopVal: Value at which to stop immediately, can be any output of
%                  LOSS that is sufficiently low for you.
%                  Default is -Inf.
%       CoolSched: Generates a new temperature from the previous one.
%                  Any function handle that takes a scalar as input and
%                  returns a smaller but positive scalar as output. 
%                  Default is @(T) (.5*T)
%      MaxConsRej: Maximum number of consecutive rejections, can be any
%                  positive number.
%                  Default is 1000.
%        MaxTries: Maximum number of tries within one temperature, can be
%                  any positive number.
%                  Default is 100.
%      MaxSuccess: Maximum number of successes within one temperature, can
%                  be any positive number.
%                  Default is 20.
%
%
%  Usage:
%     [MINIMUM,FVAL] = ANNEAL(LOSS,NEWSOL,[OPTIONS]);
%          MINIMUM is the solution which generated the smallest encountered
%          value when input into LOSS.
%          FVAL is the value of the LOSS function evaluated at MINIMUM.
%     OPTIONS = ANNEAL();
%          OPTIONS is the default options structure.

%  Reference:
%    Kirkpatrick, S., Gelatt, C.D., & Vecchi, M.P. (1983). Optimization by
%    Simulated Annealing. _Science, 220_, 671-680.

%   joachim.vandekerckhove@psy.kuleuven.be
%   $Revision: v5 $  $Date: 2006/04/26 12:54:04 $

% Copyright (c) 2010, Saloman Danaraj
% Copyright (c) 2006, Joachim Vandekerckhove
% All rights reserved.

% Open a wait bar dialog box
h_wait = waitbar(0,'Please wait...',...
    'Position',[250,320,290,50],...
    'CloseRequestFcn',@close_waitbar);

SEED = 2;
rand('state',SEED);

def = struct(...
        'CoolSched',@(T) (.5*T),...
        'InitTemp',100,...
        'MaxConsRej',1000,...
        'MaxSuccess',20,...
        'MaxTries',100,...
        'StopTemp',1e-2,...
        'StopVal',-Inf,...
        'Verbosity',2);

% Check input
if ~nargin %user wants default options, give it and stop
    minimum = def;
    return
elseif nargin<2, %user gave only objective function, throw error
    error('MATLAB:anneal:noParent','You need to input a first guess.');
elseif nargin<3, %user gave no options structure, use default
    options=def;
else %user gave all input, check if options structure is complete
    if ~isstruct(options)
        error('MATLAB:anneal:badOptions',...
            'Input argument ''options'' is not a structure')
    end
    fs = {'CoolSched','InitTemp','MaxConsRej',...
        'MaxSuccess','MaxTries','StopTemp','StopVal','Verbosity'};
    for nm=1:length(fs)
        if ~isfield(options,fs{nm}), options.(fs{nm}) = def.(fs{nm}); end
    end
end

% main settings
Tinit = options.InitTemp;        % initial temp
minT = options.StopTemp;         % stopping temp
cool = options.CoolSched;        % annealing schedule
minF = options.StopVal;
max_consec_rejections = options.MaxConsRej;
max_try = options.MaxTries;
max_success = options.MaxSuccess;
report = options.Verbosity;
k = 1;                           % constant
LB = [0.01 30e5 0 0 0 0 0];
UB = [0.4 50e5 60 30 60 30 60];
VM = 0.1 * (UB - LB);
C = 2;

% counters etc
itry = 0;
success = 0;
NI = zeros(1, 7);
NS = zeros(1, 7);
finished = 0;
consec = 0;
T = Tinit;
initenergy = loss(parent);
oldenergy = initenergy;
total = 0;
if report==2, fprintf(1,'\n  T = %7.5f, loss = %10.5f\n',T,oldenergy); end
Fval(1) = oldenergy;
Xval(1, :) = parent;

while ~finished;    
    itry = itry+1; % just an iteration counter
    current = parent; 
    
    % Update a wait bar dialog box
    waitbar((log(T/Tinit)/log(0.5)*max_try+itry)/(14*max_try),h_wait)
    
    % Stop / decrement T criteria
    if itry >= max_try || success >= max_success;
        if T < minT || consec >= max_consec_rejections;
            finished = 1;
            total = total + itry;
            break;
        else
            T = cool(T);  % decrease T according to cooling schedule
            Fval(log(T/Tinit)/log(0.5)+1) = oldenergy;
            Xval(log(T/Tinit)/log(0.5)+1, :) = parent;
            if report==2, % outputMail
                fprintf(1,'  T = %7.5f, loss = %10.5f\n',T,oldenergy);
            end
            total = total + itry;
            itry = 1;
            success = 1;
        end
    end
    
    [newparam ptr] = newsol(current, VM, LB, UB); % set new parameters
    newenergy = loss(newparam);
    
    NI(ptr) = NI(ptr) + 1;
    
    if (newenergy < minF),
        parent = newparam; 
        oldenergy = newenergy;
        break
    end
    
    if (oldenergy-newenergy > 1e-6)
        parent = newparam;
        oldenergy = newenergy;
        success = success+1;
        NS(ptr) = NS(ptr) + 1;
        consec = 0;
    else
        if (rand < exp( (oldenergy-newenergy)/(k*T) ));
            parent = newparam;
            oldenergy = newenergy;
            success = success+1;
            NS(ptr) = NS(ptr) + 1;
        else
            consec = consec+1;
        end
    end
 
    ratio = NS(ptr) / NI(ptr);
    if ratio > 0.6
        VM(ptr) = VM(ptr) * (1 + C * (ratio - 0.6) / 0.4);
    elseif ratio < 0.4
        VM(ptr) = VM(ptr) / (1 + C * (0.4 - ratio) / 0.4);
    end
    if VM(ptr) > UB(ptr) - LB(ptr)
        VM(ptr) = UB(ptr) - LB(ptr);
    end
end

minimum = Xval(Fval == min(Fval), :);
fval = min(Fval);

if report;
    fprintf(1, '\n  Initial temperature:     \t%g\n', Tinit);
    fprintf(1, '  Final temperature:       \t%g\n', T);
    fprintf(1, '  Consecutive rejections:  \t%i\n', consec);
    fprintf(1, '  Number of function calls:\t%i\n', total);
    fprintf(1, '  Total final loss:        \t%g\n', fval);
end

% When waitbar reaches max, close it.
if ishandle(h_wait)
    close(h_wait)
end

end
